{
  description = "The Flake";

  inputs = {
    nixpkgs.url = "github:nixos/nixpkgs/nixos-unstable";
    nixpkgs-stable.url = "github:nixos/nixpkgs/nixos-23.11";
    nixos-hardware.url = "github:NixOS/nixos-hardware/master";
    home-manager = {
      url = "github:nix-community/home-manager/master";
      inputs.nixpkgs.follows = "nixpkgs";
    };
    hyprland = {
      url = "github:hyprwm/Hyprland";
    };
    hyprland-plugins = {
      url = "github:hyprwm/hyprland-plugins";
      inputs.hyprland.follows = "hyprland";
    };
    eww = {
      url = "github:elkowar/eww";
      inputs.nixpkgs.follows = "nixpkgs";
    };
    emacs = {
      url = "github:nix-community/emacs-overlay";
      inputs.nixpkgs.follows = "nixpkgs";
    };
    nix-bitcoin = {
      url = "github:fort-nix/nix-bitcoin/release";
    };
    simple-nixos-mailserver = {
      url = "gitlab:simple-nixos-mailserver/nixos-mailserver/nixos-22.11";
    };
    anyrun = {
      url = "github:Kirottu/anyrun";
      inputs.nixpkgs.follows = "nixpkgs";
    };
    libre-presenter = {
      url = "gitlab:chriscochrun/lumina";
    };
    # nixified-ai = {
    #   url = "github:nixified-ai/flake";
    #   inputs.nixpkgs.follows = "nixpkgs";
    # };
  };

  outputs = { nixpkgs, nixpkgs-stable, home-manager, hyprland, nixos-hardware, emacs, nix-bitcoin, libre-presenter, anyrun, ... }:
    let
      system = "x86_64-linux";
      username = "chris";
      pkgsForSystem = import nixpkgs {
        inherit system;
        config = { allowUnfree = true; };
        overlays = [
          emacs.overlays.default
        ];
      };

      lib = nixpkgs.lib;

    in  {
      nixosConfigurations = {
        syl = lib.nixosSystem {
          inherit system;
          pkgs = pkgsForSystem;
          modules = [
            ./systems/syl/configuration.nix
            home-manager.nixosModules.home-manager
            hyprland.nixosModules.default
            nixos-hardware.nixosModules.framework-11th-gen-intel
            # libre-presenter.defaultPackage
            {
              home-manager.useGlobalPkgs = true;
              home-manager.useUserPackages = true;
              home-manager.users.chris = import ./home/home.nix;
            }
          ];
        };
        kaladin = lib.nixosSystem {
          inherit system;
          pkgs = pkgsForSystem;
          modules = [
            ./systems/kaladin/configuration.nix
            home-manager.nixosModules.home-manager
            hyprland.nixosModules.default
            nixos-hardware.nixosModules.common-gpu-amd
            {
              home-manager.useGlobalPkgs = true;
              home-manager.useUserPackages = true;
              home-manager.users.chris = import ./home/home.nix;
            }
          ];
        };
        dalinar = nixpkgs-stable.lib.nixosSystem {
          inherit system;
          modules = [
            ./systems/dalinar/configuration.nix
            nix-bitcoin.nixosModules.default
            # simple-nixos-mailserver.nixosModules.default
          ];
        };
        kohlin = nixpkgs-stable.lib.nixosSystem {
          inherit system;
          modules = [
            ./systems/kohlin/configuration.nix
            nix-bitcoin.nixosModules.default
          ];
        };
      };
    };
}
